<?php

declare(strict_types=1);

namespace spec\Dedi\SyliusVoucherPlugin\Specification;

use Dedi\SyliusVoucherPlugin\Model\OrderInterface;
use Dedi\SyliusVoucherPlugin\Specification\IsOrderFullyPaidUsingVoucherSpecification;
use PhpSpec\ObjectBehavior;

class IsOrderFullyPaidUsingVoucherSpecificationSpec extends ObjectBehavior
{
    function it_is_initializable()
    {
        $this->shouldHaveType(IsOrderFullyPaidUsingVoucherSpecification::class);
    }

    function it_returns_true_for_fully_paid_order_using_vouchers(
        OrderInterface $order
    ) {
        $order->getTotal()->willReturn(0);
        $order->hasVouchers()->willReturn(true);

        $this->__invoke($order)->shouldReturn(true);
    }

    function it_returns_false_for_non_fully_paid_order_using_vouchers(
        OrderInterface $order
    ) {
        $order->getTotal()->willReturn(10);
        $order->hasVouchers()->willReturn(true);

        $this->__invoke($order)->shouldReturn(false);
    }

    function it_returns_false_for_fully_paid_order_not_using_vouchers(
        OrderInterface $order
    ) {
        $order->getTotal()->willReturn(0);
        $order->hasVouchers()->willReturn(false);

        $this->__invoke($order)->shouldReturn(false);
    }

    function it_returns_false_for_order_not_paid_and_not_using_voucher(
        OrderInterface $order
    ) {
        $order->getTotal()->willReturn(10);
        $order->hasVouchers()->willReturn(false);

        $this->__invoke($order)->shouldReturn(false);
    }
}
