<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Controller\Action;

use Dedi\SyliusVoucherPlugin\Applicator\VoucherApplicatorInterface;
use Dedi\SyliusVoucherPlugin\Model\OrderInterface;
use Sylius\Component\Order\Context\CartContextInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\Session\Session;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Webmozart\Assert\Assert;

final class RemoveVoucherFromOrderAction
{
    public function __construct(
        private readonly CartContextInterface $cartContext,
        private readonly RequestStack $requestStack,
        private readonly UrlGeneratorInterface $urlGenerator,
        private readonly VoucherApplicatorInterface $voucherApplicator,
    ) {
    }

    public function __invoke(Request $request): Response
    {
        /** @var OrderInterface|null $order */
        $order = $this->cartContext->getCart();
        Assert::notNull($order);

        $this->voucherApplicator->remove($order, $request->attributes->get('voucher'));

        $request = $this->requestStack->getCurrentRequest();
        /** @var Session $session */
        $session = $request->getSession();

        $session->getFlashBag()->add('success', 'dedi_sylius_voucher.voucher_removed');

        return new RedirectResponse($this->urlGenerator->generate('sylius_shop_checkout_select_payment'));
    }
}
