<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\DependencyInjection;

use Dedi\SyliusVoucherPlugin\Checker\VoucherRuleInterface;
use Exception;
use Sylius\Bundle\ResourceBundle\DependencyInjection\Extension\AbstractResourceExtension;
use Symfony\Component\Config\FileLocator;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Loader\XmlFileLoader;

final class DediSyliusVoucherExtension extends AbstractResourceExtension
{
    /**
     * @throws Exception
     */
    public function load(array $config, ContainerBuilder $container): void
    {
        $config = $this->processConfiguration($this->getConfiguration([], $container), $config);
        $loader = new XmlFileLoader($container, new FileLocator(__DIR__ . '/../Resources/config'));
        $loader->load('services.xml');

        $this->registerResources('dedi_sylius_voucher', $config['driver'], $config['resources'], $container);

        $settingsDefinition = $container->getDefinition('dedi_sylius_voucher.settings');
        $settingsDefinition->replaceArgument('$enabled', $config['enabled']);
        $settingsDefinition->replaceArgument('$useNoRestrictionRule', $config['use_no_restriction_rule']);
        $settingsDefinition->replaceArgument('$codeLength', $config['code_length']);

        if ($config['use_no_restriction_rule']) {
            $loader = new XmlFileLoader($container, new FileLocator(__DIR__ . '/../Resources/config/services'));

            $loader->load('checker_no_restriction_rule.xml');
        }

        $container->registerForAutoconfiguration(VoucherRuleInterface::class)
            ->addTag('dedi_sylius_voucher.eligibility.rule');
    }
}
