<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\DependencyInjection\Settings\Twig;

use Dedi\Contracts\SyliusPluginSettings\SettingsProviderInterface;
use Dedi\SyliusVoucherPlugin\DependencyInjection\Settings\SettingsInterface;
use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;
use Webmozart\Assert\Assert;

class SettingsCheckerExtension extends AbstractExtension
{
    public function __construct(private readonly SettingsProviderInterface $settingsProvider)
    {
    }

    public function getFunctions(): array
    {
        return [
            new TwigFunction('dedi_sylius_voucher_check', [$this, 'check']),
            new TwigFunction('dedi_sylius_voucher_is_enabled', [$this, 'isEnabled']),
        ];
    }

    public function check(string $setting): mixed
    {
        $settings = $this->settingsProvider->provide();

        Assert::isInstanceOf($settings, SettingsInterface::class);

        switch ($setting) {
            case 'enabled':
                return $settings->isEnabled();
            case 'use_no_restriction_rule':
                return $settings->isUseNoRestrictionRule();
            case 'code_length':
                return $settings->getCodeLength();
            default:
                throw new \InvalidArgumentException(sprintf('Unrecognized Voucher Plugin setting %s', $setting));
        }
    }

    public function isEnabled(): bool
    {
        return (bool) $this->check('enabled');
    }
}
