<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\EventListener;

use Dedi\SyliusVoucherPlugin\Model\VoucherConfigurationInterface;
use Sylius\Bundle\ResourceBundle\Event\ResourceControllerEvent;
use Sylius\Component\Resource\Exception\UnexpectedTypeException;
use Sylius\Component\Resource\Repository\RepositoryInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

final class DefaultVoucherConfigurationSubscriber implements EventSubscriberInterface
{
    private readonly RepositoryInterface $voucherConfigurationRepository;

    public function __construct(RepositoryInterface $voucherConfigurationRepository)
    {
        $this->voucherConfigurationRepository = $voucherConfigurationRepository;
    }

    public static function getSubscribedEvents(): array
    {
        return [
            'dedi_sylius_voucher.voucher_configuration.pre_update' => 'preUpdate',
        ];
    }

    public function preUpdate(ResourceControllerEvent $event): void
    {
        $voucherConfiguration = $event->getSubject();
        if (!$voucherConfiguration instanceof VoucherConfigurationInterface) {
            throw new UnexpectedTypeException($voucherConfiguration, VoucherConfigurationInterface::class);
        }

        if ($voucherConfiguration->isDefault()) {
            $currentDefault = $this->voucherConfigurationRepository->findOneBy(['default' => true]);
            if ($currentDefault instanceof VoucherConfigurationInterface && $currentDefault !== $voucherConfiguration) {
                $currentDefault->setDefault(false);
            }
        }
    }
}
