<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\EventListener;

use Dedi\SyliusVoucherPlugin\Model\VoucherInterface;
use Sylius\Bundle\ResourceBundle\Event\ResourceControllerEvent;
use Sylius\Component\Resource\Exception\UnexpectedTypeException;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

final class VoucherDeletionSubscriber implements EventSubscriberInterface
{
    public static function getSubscribedEvents(): array
    {
        return [
            'dedi_sylius_voucher.voucher.pre_delete' => 'onVoucherPreDelete',
        ];
    }

    /**
     * Prevent voucher code deletion if it not deletable
     */
    public function onVoucherPreDelete(ResourceControllerEvent $event): void
    {
        $voucher = $event->getSubject();

        if (!$voucher instanceof VoucherInterface) {
            throw new UnexpectedTypeException(
                $voucher,
                VoucherInterface::class,
            );
        }

        if (!$voucher->isDeletable()) {
            $event->stop('dedi_sylius_voucher.voucher.delete_error');
        }
    }
}
