<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Factory;

use Dedi\SyliusVoucherPlugin\Generator\VoucherCodeGeneratorInterface;
use Dedi\SyliusVoucherPlugin\Model\VoucherInterface;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Core\Model\CustomerInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Core\Model\OrderItemUnitInterface;
use Sylius\Component\Resource\Factory\FactoryInterface;
use Webmozart\Assert\Assert;

final class VoucherFactory implements VoucherFactoryInterface
{
    private readonly FactoryInterface $decoratedFactory;

    private readonly VoucherCodeGeneratorInterface $voucherCodeGenerator;

    public function __construct(FactoryInterface $decoratedFactory, VoucherCodeGeneratorInterface $voucherCodeGenerator)
    {
        $this->decoratedFactory = $decoratedFactory;
        $this->voucherCodeGenerator = $voucherCodeGenerator;
    }

    public function createNew(): VoucherInterface
    {
        /** @var VoucherInterface $voucher */
        $voucher = $this->decoratedFactory->createNew();
        $voucher->setCode($this->voucherCodeGenerator->generate());

        return $voucher;
    }

    public function createForChannel(ChannelInterface $channel): VoucherInterface
    {
        $voucher = $this->createNew();
        $voucher->setChannel($channel);

        return $voucher;
    }

    public function createFromOrderItemUnit(OrderItemUnitInterface $orderItemUnit): VoucherInterface
    {
        /** @var OrderInterface|null $order */
        $order = $orderItemUnit->getOrderItem()->getOrder();

        Assert::isInstanceOf($order, OrderInterface::class);

        /** @var ChannelInterface|null $channel */
        $channel = $order->getChannel();

        Assert::isInstanceOf($channel, ChannelInterface::class);

        $currencyCode = $order->getCurrencyCode();
        Assert::notNull($currencyCode);

        /** @var CustomerInterface|null $customer */
        $customer = $order->getCustomer();
        Assert::isInstanceOf($customer, CustomerInterface::class);

        $voucher = $this->createNew();
        $voucher->setCustomer($customer);
        $voucher->setOrderItemUnit($orderItemUnit);
        $voucher->setChannel($channel);
        $voucher->setAmount($orderItemUnit->getTotal());
        $voucher->setCurrencyCode($currencyCode);
        $voucher->setInitialAmount($orderItemUnit->getTotal());
        $voucher->disable();

        return $voucher;
    }
}
