<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Generator;

use Dedi\Contracts\SyliusPluginSettings\SettingsProviderInterface;
use Dedi\SyliusVoucherPlugin\DependencyInjection\Settings\SettingsInterface;
use Dedi\SyliusVoucherPlugin\Repository\VoucherRepositoryInterface;
use Exception;
use Webmozart\Assert\Assert;

final class VoucherCodeGenerator implements VoucherCodeGeneratorInterface
{
    private readonly VoucherRepositoryInterface $voucherRepository;

    public function __construct(VoucherRepositoryInterface $voucherRepository, private readonly SettingsProviderInterface $settingsProvider)
    {
        $this->voucherRepository = $voucherRepository;
    }

    /**
     * @throws Exception
     */
    public function generate(): string
    {
        $settings = $this->settingsProvider->provide();

        Assert::isInstanceOf($settings, SettingsInterface::class);

        $length = $settings->getCodeLength();
        do {
            // if we didn't remove the 'hard to read' characters we would only have to
            // generate codeLength / 2 bytes because hex uses two characters to represent one byte
            $code = bin2hex(random_bytes($length));
            $code = preg_replace('/[01]/', '', $code); // remove hard to read characters
            $code = mb_strtoupper(mb_substr($code, 0, $length));
        } while (mb_strlen($code) !== $length || $this->exists($code));

        return $code;
    }

    private function exists(string $code): bool
    {
        return null !== $this->voucherRepository->findOneByCode($code);
    }
}
