<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Model;

use Sylius\Component\Channel\Model\ChannelInterface as SyliusChannelInterface;
use Sylius\Component\Locale\Model\LocaleInterface;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Webmozart\Assert\Assert;

/**
 * @UniqueEntity(
 *     fields={"channel", "locale"},
 *     groups={"sylius", "default"},
 *     errorPath="channel",
 *     message="Voucher channel configuration is already defined for this locale."
 * )
 */
class VoucherChannelConfiguration implements VoucherChannelConfigurationInterface
{
    protected ?int $id;

    protected ?SyliusChannelInterface $channel;

    protected ?LocaleInterface $locale;

    protected ?VoucherConfigurationInterface $configuration;

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getChannel(): ?SyliusChannelInterface
    {
        return $this->channel;
    }

    public function setChannel(?SyliusChannelInterface $channel): void
    {
        Assert::notNull($channel);
        $this->channel = $channel;
    }

    public function getLocale(): ?LocaleInterface
    {
        return $this->locale;
    }

    public function setLocale(LocaleInterface $locale): self
    {
        $this->locale = $locale;

        return $this;
    }

    public function getConfiguration(): ?VoucherConfigurationInterface
    {
        return $this->configuration;
    }

    public function setConfiguration(?VoucherConfigurationInterface $configuration): self
    {
        $this->configuration = $configuration;

        return $this;
    }
}
