<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Twig;

use Dedi\SyliusVoucherPlugin\Checker\VoucherCheckerContextInterface;
use Dedi\SyliusVoucherPlugin\Repository\VoucherRepositoryInterface;
use Sylius\Component\Core\Model\ShopUserInterface;
use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;

class CheckUserVoucherEnabledExtension extends AbstractExtension
{
    private readonly VoucherRepositoryInterface $voucherRepository;

    private readonly VoucherCheckerContextInterface $checkerContext;

    public function __construct(
        VoucherRepositoryInterface $voucherRepository,
        VoucherCheckerContextInterface $checkerContext,
    ) {
        $this->voucherRepository = $voucherRepository;
        $this->checkerContext = $checkerContext;
    }

    public function getFunctions(): array
    {
        return [
            new TwigFunction('dedi_voucher_check_user_enabled_voucher', [$this, 'isEnabled']),
        ];
    }

    public function isEnabled(ShopUserInterface $shopUser = null): array
    {
        if (null !== $shopUser) {
            return $this->checkerContext->getVouchersEligible(
                $this->voucherRepository->findEnabledByUser($shopUser->getCustomer()),
            );
        }

        return [];
    }
}
